"""
.. versionadded:: 0.2
.. versionchanged:: 0.4

This function generates steps according to given sequence of values and
given width of steps. Function also can repeat given
sequence according required number of repeats, or desired lenght of data.

Example Usage
==================

Usage is as simple as

.. code-block:: python

    import signalz
    x = signalz.steps(2, [1, 2, 3], repeat=2)
    
where the `2` stands for step width, `[1, 2, 3]` are values for steps and
`repeat=2` make the sequence of steps repeat twice. So the returned output is

>>> [1 1 2 2 3 3 1 1 2 2 3 3]


Function Documentation
======================================
"""
import numpy as np

from signalz.misc import check_type_or_raise

def steps(step_width, values, repeat=1, size=None):
    """
    This function generates steps from given values.
    
    **Args:**
    
    * `step_width` - desired width of every step (int)
    
    * `values` - values for steps (1d array) 
    
    **Kwargs:**
         
    * `repeat` - number of step sequence repetions (int), this variable is used,
      if the `size` is not defined
    
    * `size` - size of output data in samples (int), if the `size` is used,
      the `repeat` is ignored.
    
    **Returns:**
    
    * array of values representing desired steps (1d array)
    """
    check_type_or_raise(step_width, int, "step width")
    check_type_or_raise(repeat, int, "repeat")
    try:
        values = np.array(values)
    except:
        raise ValueError('Values must be a numpy array or similar.') 
    # generate steps
    x = np.repeat(values, step_width)          
    if size is None:
        # repeat according to the desired repetions
        x_full = np.tile(x, repeat)      
    else:
        check_type_or_raise(size, int, "size")
        # repeat till size is reached and crop the data to match size
        repeat = int(np.ceil(size / float(len(x))))
        x_full = np.tile(x, repeat)[:size]
    return x_full








"""
.. versionadded:: 0.4

This function generates Brownian noise series. The noise is produced by
integration of white noise (gaussian or uniform).

This function uses
`numpy.random.normal` and `numpy.random.uniform`

Example Usage
==================

The following example produce 1000 samples of brownian noise starting in
value 10 (`start=10`). The noise is produced by integration of white gaussian
noise (`source="gaussian"`) with standard deviation of 1 (`std=1`).
To keep in noise in some reasonable range,
it is used 10% leaky integration (`leak=0.1`).

.. code-block:: python

    import signalz
    x = signalz.brownian_noise(1000, leak=0.1, start=10, std=1, source="gaussian")

Function Documentation
======================================
"""
import numpy as np

from signalz.misc import check_type_or_raise
from signalz.generators.gaussian_white_noise import gaussian_white_noise
from signalz.generators.uniform_white_noise import uniform_white_noise

def brownian_noise(n, leak=0., start=0, std=1., source="gaussian"):
    """
    This function produces Browninan noise.

    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**
    
    * `leak` - leakage during integration (float), this should prevent signal
      from wander off. Possible value is `0 <= leak < 1`
      
    * `start` - offset on the start (float)

    * `std` - standard deviation of source white noise (float), in case of
      uniform distribution it is the difference between min and max value. 
    
    * `source` - distribution of source white noise (str). Options are
      `gaussian` or `uniform`
    
    **Returns:**
    
    * vector of values representing the noise (1d array)
    """
    # check inputs
    check_type_or_raise(n, int, "n")
    check_type_or_raise(leak, float, "leak")
    if not 0. <= leak < 1:
        raise ValueError("Leak must be between 0. and 1.")
    # generate white noise
    if source == "gaussian":
        x = np.random.normal(0, std, n)
    elif source == "uniform":
        x = np.random.uniform(-std/2., std/2., n)
    else:
        raise ValueError("Source must be gaussian or uniform")
    # add offset
    x[0] = start
    # integrate the white noise    
    for i in range(1,n):
        x[i] += x[i-1]*(1-leak)
    return x



"""
.. versionadded:: 0.5

This function generates Levy walk by interpolation of Levy flight.

The Levy distribution is defined by two parameters :math:`\\alpha`
and :math:`\\beta`. The Gaussian distribution is special case of 
Levy distribution with :math:`\\alpha=2` and :math:`\\beta=0`.

This function uses :ref:`generators-levy_noise`.

Example Usage
==================

The following example produce 1000 samples of Levy walk created from
500 samples of Levy noise (`ns=500`) located (mean value) at 0 (`position`),
with characteristic exponent index of 1.4 (`alpha`),
skeewness of 0 (`beta`) and diffusion of 1. (`sigma`). 


.. code-block:: python

    import signalz
    x = signalz.levy_walk(1000, ns=500, alpha=1.4, beta=0., sigma=1., position=0)

Function Documentation
======================================
"""
import numpy as np

from signalz.misc import check_type_or_raise
import signalz

def levy_walk(n, ns=0, alpha=2., beta=1., sigma=1., position=0.):
    """
    This function produces Levy walk.
    
    **Args:**
    
    * `n` - length of the output data (int) -
      how many samples will be on output
    
    * `ns` - number of points (int) in original noise before interpolation,
      this number cannot be higher than desired length of data;
      if it is set to 0, then `ns=n` is used; this number is related to number
      of direction changes in resulting walk
    
    **Kwargs:**
    
    * `alpha` - characteristic exponent index of used Levy noise
      (float) in range `0<alpha<2`
      
    * `beta` - skeewness of used Levy noise (float) in range `-1<beta<1`

    * `sigma` - diffusion of used Levy noise (float);
      in case of gaussian distribution it is
      standard deviation

    * `position` - position parameter (float) of used Levy noise
    
    **Returns:**
    
    * vector of values representing the walk (1d array)
    """
    # check params
    check_type_or_raise(ns, int, "ns")
    check_type_or_raise(n, int, "n")
    if ns == 0:
        ns = n
    elif ns > n:
        raise ValueError("Argument ns cannot be higher than argument n")
    # generate levy random variable
    x = signalz.levy_noise(ns, alpha, beta, sigma, position)
    # interpolate new points
    xt = np.cumsum(np.abs(x))  
    xf = np.cumsum(x)
    t = np.linspace(0, xt[-1], n)
    return np.interp(t, xt, xf)
        
        




"""
.. versionadded:: 0.1
.. versionchanged:: 0.2

This function generates data as autoregressive model (AR model) according to
following equation 

:math:`x(k) = c + \sum\limits_{n=1}^{\infty} a_{i} x(k-i) + v(k)`,

where :math:`k` is discrete time index, :math:`c` is an constant, 
:math:`a_i` is parameters of model and :math:`v(k)` is noise.

The AR model is also known as stochastic difference equation.

Example Usage
==================

Let us consider AR model as follows

:math:`x(k) = 1.79 x(k-1) - 1.85 x(k-2) + 1.27 x(k-3) - 0.41 x(k-4) + v(k)`.

This AR can be simulated with following code (1000 samples).

.. code-block:: python

    import signalz

    # AR model parameters
    a = [-0.41, 1.27, -1.85, 1.79]

    # number of samples
    N = 1000
    
    # get AR data
    x = signalz.autoregressive_model(N, a, noise="white")

The next example shows, how to introduce parameter changes during the data
generation.
    
.. code-block:: python

    import signalz

    # number of samples
    N = 1000

    # AM model default parameters
    a = [-0.41, 1.27, -1.85, 1.79]

    # parameters for all samples
    A = np.ones((N,4))
    A = A*a

    # change of parameters starting from time index 500
    A[500:] = a + np.array([0.01, 0.02, 0.01, -0.02])

    # get AR data
    x = signalz.autoregressive_model(N, A, noise="white")
    

Function Documentation
======================================
"""
import numpy as np
from signalz.generators.gaussian_white_noise import gaussian_white_noise

import signalz
    
def autoregressive_model(n, a, const=0, noise="white", initials="none"):
    """
    Autoregressive model (stochastic difference equation)
    
    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    * `a` - coefficients of the model (1d array, 2d array), in case of a vector
      are used the same parameters for whole generation. In case of a matrix,
      every row represents parameters for one time sample.
    
    **Kwargs:**

    * `const` - constant of the model (float), default is 0
    
    * `noise` - model input (str or 1d array), default is "white",
      possible options are:
        
      * `"white"` - input noise has zero mean value and unit standard deviation
        
      * `"none"` - input noise are zeros
        
      * manually created array of inputs of length `n` 
         
    * `initials` - initial values (1d array) of the same size as `a`
    
    **Returns:**
    
    * `x` - output of the AR model (1d array)    
    """
    a = np.array(a)
    taps = a.shape[-1]
    a = signalz.matrixize_input(a, n)
    # handle noise
    if type(noise) == str:
        if noise == "none":
            noise = np.zeros(n)
        elif noise == "white":
            noise = gaussian_white_noise(n)
    else:
        try:
            noise = np.array(noise)
        except:
            raise ValueError('Noise is not numpy array or similar.')
        if not len(noise) == n:
            raise ValueError('Noise array is not same length as n.')
    x = np.zeros(n)
    # handle initials 
    if initials == "random":
        x[:taps] = gaussian_white_noise(taps)
    elif initials == "none":
        pass   
    else:
        if not len(a) == len(initials):
            raise ValueError('The length of initials values and a must agree.')
        x[:taps] = initials
    # simulate system
    for k in range(taps,n-1):
        x[k+1] = const + np.dot(a[k], x[k+1-taps:k+1]) + noise[k+1]
    return x  
    





"""
.. versionadded:: 0.1
.. versionchanged:: 0.2

This function generates data according a discrete realization of Mackey-Glass 
equation as follows

:math:`x_{k+1} = c \cdot x_k + \\frac{\large{a \cdot x_{k-d}}}{b + x^{e}_{k-d}}`

The original Mackey-Glass equation :cite:`glass2010mackey` is the nonlinear
time delay differential equation.

Example Usage
===============

In this example is simulated 1000 samples with arguments that cause
chaotic behaviour.

.. code-block:: python

    N = 1000
    x = signalz.mackey_glass(N, a=0.2, b=0.8, c=0.9, d=23, e=10, initial=0.1)
    
The parameters  `a`, `b`, `c`, `d`, `e` can be a scalar or a vector. In case
of a vector, every item represents parameter for one sample.

References
============

.. bibliography:: mackey_glass.bib
    :style: plain

Function Documentation
======================================
"""
import numpy as np

import signalz

def mackey_glass(n, a=0.2, b=0.8, c=0.9, d=23, e=10, initial=0.1):
    """
    Mackey-Glass discrete equation.
    
    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output

    **Kwargs:**
    
    Parameters  `a`, `b`, `c`, `d`, `e` can be a scalar or a vector. In case
    of a vector, every item represents parameter for one sample.

    * `a` - parameter of the model (float, 1d array), default is 0.2
    
    * `b` - parameter of the model (float, 1d array), default is 0.8
    
    * `c` - parameter of the model (float, 1d array), default is 0.9
    
    * `d` - time delay of the model (int, 1d array), default is 23
    
    * `e` - parameter of the model (float, 1d array), default is 10
    
    * `initial` - initial value (float), default is 0.1
    
    **Returns:**
    
    * `x` - output of Mackey-Glass equation   
    """    
    x = np.zeros(n)
    x[0] = initial
    d = int(d)
    a = signalz.vectorize_input(a, n)
    b = signalz.vectorize_input(b, n)
    c = signalz.vectorize_input(c, n)
    d = signalz.vectorize_input(d, n).astype("int")
    e = signalz.vectorize_input(e, n)
    for k in range(0,n-1):
        x[k+1]  = c[k]*x[k] + ( (a[k]*x[k-d[k]]) / (b[k] + ( x[k-d[k]]**e[k])) )   
    return x













"""
.. versionadded:: 0.3

This function simulates logistic map equation. This logisti map is described
by following equation

:math:`x(k+1) = r x(k) (1 - x(k))`,

where :math:`x` is population (between 0 and 1) and :math:`x` is parameter
denoted as value of interest.

The logistic map can result in logistic behaviour for some values of :math:`r`.

For more information see great page about
`logistic map on Wikipedia <https://en.wikipedia.org/wiki/Logistic_map>`_

Example Usage
===============

Simple example

.. code-block:: python

    N = 200 # number of samples
    x = signalz.logistic_map(N, r=3)

Function Documentation
======================================
"""
import numpy as np

def logistic_map(n, r=4, initial=0.5):
    """
    Logist map equation

    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**
    
    * `r` - function parameter (float)
    
    * `initial` - initial value (float), default is 0.1,
      should be in range from 0 to 1
    
    **Returns:**
    
    * simulation output - vector of numbers between 0 and 1
    """
    x = np.zeros(n)
    x[0] = initial
    for k in range(n-1):
        x[k+1] = r*x[k]*(1-x[k])
    return x





"""
.. versionadded:: 0.2
.. versionchanged:: 0.4

This function generates random steps according to given width of steps and
desired number of steps.

Example Usage
==================

Simplest example (10 steps with width 50 samples, normal distribution
with unit standard deviation and zero mean) follows.

.. code-block:: python

    import signalz
    x = signalz.random_steps(50, 10)
       
More complicated example with normal distribution (standard deviation and
mean value is changed) follows.

.. code-block:: python

    import signalz
    x = signalz.random_steps(50, steps_count=10, distribution="normal", std=30, mean=-10)
    
Another example, this time the size of the data is not requested by number of
steps, but by number of samples (`size=500`). 

.. code-block:: python

    import signalz
    x = signalz.random_steps(50, size=500)

Function Documentation
======================================
"""
import numpy as np

from signalz.misc import check_type_or_raise
import signalz

def random_steps(step_width, steps_count=10, size=None, distribution="normal",
    maximum=1, minimum=0, std=1, mean=0):
    """
    This function generates random steps.
    
    **Args:**
    
    * `step_width` - desired width of every step (int)
       
    **Kwargs:**
    
    * `steps_count` - desired number steps (int), this variable is used,
      if the `size` is not defined
    
    * `size` - lenght of desired output in samples (int),
      if this variable is defined,
      it determines the size of data instead of `steps_count`
             
    * `distribution` - distribution of random numbers (str), Options are
      `normal` and `uniform`.
      
    * `maximum` - maximal value for steps (float), this value is used
      in case of uniform distribution.
    
    * `minimum` - minimal value for steps (float), this value is used
      in case of uniform distribution.
    
    * `std` - standard deviation of random variable (float), this value
      is used in case of gaussian (normal) distribution.
      
    * `mean` - mean value of random variable (float), this value
      is used in case of gaussian (normal) distribution.
    
    **Returns:**
    
    * vector of values representing desired steps (1d array)
    """ 
    # check values
    check_type_or_raise(steps_count, int, "steps count")
    check_type_or_raise(step_width, int, "step width")
    # get correct number of steps if size is defined
    if not size is None:
        check_type_or_raise(size, int, "size")
        steps_count = int(np.ceil(size / float(step_width)))
    # generate random values
    if distribution in ["normal", "gaussian"]:
        values = signalz.gaussian_white_noise(steps_count, offset=mean, std=std)   
    elif distribution == "uniform":
        values = signalz.uniform_white_noise(steps_count, minimum=minimum,
            maximum=maximum)
    # generate steps
    x = signalz.steps(step_width, values)
    if size is None:
        return x
    else:
        return x[:size]
        



"""
.. versionadded:: 0.5

This function generates random variable with Levy alpha-stable distribution
(also reffered just as stable distribution).

The Levy distribution is defined by two parameters :math:`\\alpha`
and :math:`\\beta`. The Gaussian distribution is special case of 
Levy distribution with :math:`\\alpha=2` and :math:`\\beta=0`.

Notes about Implementation
==============================
The implemented algorithm is known as Chambers-Mallows-Stuck method
:cite:`weron1996chambers`. Using two random variables (one with exponential
distribution and one with uniform distribution). The input random variables
are obtained with `numpy.random.exponentinal` and `numpy.random.uniform`.

Example Usage
==================

The following example produce 1000 samples of Levy noise located (mean value)
at -2 (`position`), with characteristic exponent index of 1.5 (`alpha`),
skeewness of 1 (`beta`) and diffusion of 1. (`sigma`).

.. code-block:: python

    import signalz
    x = signalz.levy_noise(1000, alpha=1.5, beta=0.5, sigma=1., position=-2)

References
============

.. bibliography:: levy_noise.bib
    :style: plain

Function Documentation
======================================
"""
import numpy as np

from signalz.misc import check_type_or_raise

def levy_noise(n, alpha=2., beta=1., sigma=1., position=0.):
    """
    This function produces Levy noise.
    
    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**
    
    * `alpha` - characteristic exponent index (float) in range `0<alpha<2`
      
    * `beta` - skeewness (float) in range `-1<beta<1`

    * `sigma` - diffusion (float), in case of gaussian distribution it is
      standard deviation

    * `position` - position parameter (float)
    
    **Returns:**
    
    * vector of values representing the noise (1d array)
    """
    # correct the inputs or throw error
    alpha = float(alpha)
    beta = float(beta)
    check_type_or_raise(n, int, "n")
    if not 0. <= alpha <= 2.:
        raise ValueError("Alpha must be between 0 and 2")
    if not -1. <= beta <= 1.:
        raise ValueError("Beta must be between -1 and 1")
    # generate random variables
    v = np.random.uniform(-0.5 * np.pi, 0.5 * np.pi, n)
    w = np.random.exponential(1, n)
    # convert random variables to levy noise
    if alpha == 1.:
        arg1 = (0.5 * np.pi) + (beta * v)
        arg2 = w * np.cos(v)
        arg3 = 0.5 * np.pi * beta * sigma + np.log(sigma)
        x = 2 / np.pi * (arg1 * np.tan(v) - (beta * np.log(arg2 / arg1)))
        return (sigma * x) + arg3 + position
    else:
        arg1 = 0.5 * np.pi * alpha
        b_ab = np.arctan(beta * np.tan(arg1)) / alpha
        s_ab = (1 + (beta**2) * np.tan(arg1)**2)**(1/(2.*alpha))
        arg2 = alpha * (v + b_ab)
        n1 = np.sin(arg2)
        d1 = np.cos(v)**(1/alpha)
        n2 = np.cos(v - arg2)
        x = s_ab * (n1/d1) * (n2/w)**((1-alpha)/alpha)
        return (sigma * x) + position



"""
.. versionadded:: 0.1

This function generates sinus waves with given period, amplitude and offset.
It is also possible to use delay defined in samples to crop the first wave.

Example Usage
===============

Simple example

.. code-block:: python

    N = 200 # number of samples
    x = signalz.sinus(N)

Function Documentation
======================================
"""
import numpy as np

def sinus(n, period=100, amplitude=2, offset=0, delay=0):
    """
    Sinus waves generator.

    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**
    
    * `period` - length of one sinus wave period (1 / frequency)
    
    * `amplitude` - difference between minimal and maximal value of waves 

    * `offset` - mean value (float)
         
    * `delay` - how many samples should be croped from first wave (does not
      influence full length of data)
    
    **Returns:**
    
    * vector of sinus waves (1d array)   
    """
    t = np.arange(n) + delay
    return np.sin(t*2*np.pi/float(period))*0.5*amplitude + offset





"""
Current version: |version| (:ref:`changelog`)

This library is designed to simplify tasks of synthetic data generation
with Python.
For code optimisation,
this library uses `Numpy <http://www.numpy.org/>`_ for array operations.

.. toctree::
    :maxdepth: 2
    
    index

License
===============

This project is under `MIT License <https://en.wikipedia.org/wiki/MIT_License>`_.

Instalation
====================
With `pip <https://pypi.python.org/pypi/pip>`_ from terminal:
``$ pip install signalz``

Or download you can download the source codes from Github
(`link <https://github.com/matousc89/signalz>`_)

The User Quide
=====================

Here is complete documentation for all implemented functions.

.. cssclass:: funcitem

    * :ref:`generators-sinus`

.. cssclass:: functag

    :ref:`tags-goniometric` 
    

.. cssclass:: funcitem

    * :ref:`generators-cosinus`

.. cssclass:: functag

    :ref:`tags-goniometric` 
    

.. cssclass:: funcitem

    * :ref:`generators-steps`

.. cssclass:: functag

    :ref:`tags-steps` 
    

.. cssclass:: funcitem

    * :ref:`generators-random_steps`

.. cssclass:: functag

    :ref:`tags-steps`, :ref:`tags-random` 
    

.. cssclass:: funcitem

    * :ref:`generators-mackey_glass`

.. cssclass:: functag

    :ref:`tags-chaotic` 
    

.. cssclass:: funcitem

    * :ref:`generators-gaussian_white_noise`

.. cssclass:: functag

    :ref:`tags-random`, :ref:`tags-noise` 
    

.. cssclass:: funcitem

    * :ref:`generators-uniform_white_noise`

.. cssclass:: functag

    :ref:`tags-random`, :ref:`tags-noise`

    
.. cssclass:: funcitem

    * :ref:`generators-levy_noise`

.. cssclass:: functag

    :ref:`tags-random`, :ref:`tags-noise`


.. cssclass:: funcitem

    * :ref:`generators-brownian_noise`

.. cssclass:: functag

    :ref:`tags-random`, :ref:`tags-noise`


.. cssclass:: funcitem

    * :ref:`generators-levy_flight`

.. cssclass:: functag

    :ref:`tags-random`


.. cssclass:: funcitem

    * :ref:`generators-levy_walk`

.. cssclass:: functag

    :ref:`tags-random`
        

.. cssclass:: funcitem

    * :ref:`generators-autoregressive_model`

.. cssclass:: functag

    :ref:`tags-autoregressive` 
    

.. cssclass:: funcitem

    * :ref:`generators-logistic_map`

.. cssclass:: functag

    :ref:`tags-population_model`, :ref:`tags-chaotic`  



Contact
=====================

By email: matousc@gmail.com


Changelog
======================

For informations about versions and updates see :ref:`changelog`.

Indices and tables
===========================
* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`

"""

from signalz.generators import *
from signalz.misc import *




import numpy as np

def vectorize_input(x, n):
    """
    Test if input parameter is a vector, if not, create array of desired lenght
    containing only the copies of input parameter.
    """
    if hasattr(x, '__len__') and (not isinstance(x, str)):
        pass
    else:    
        x = np.ones(n)*x  
    return x  
    
def matrixize_input(x, n):
    """
    Test if input vector 1d or a matrix, if it is matrix, then create matrix
    of desired lenght containing only the copies of input vector.
    """
    if len(x.shape) == 1:
        x = np.tile(x, (n, 1))
    return x  
    
def check_type_or_raise(obj, expected_type, obj_name):
    """
    This fuction raises error if the object does not have expected type.
    """
    if not isinstance(obj, expected_type):
        raise TypeError(
            "'{}' must be {}, not {}".format(
                obj_name,
                expected_type.__name__,
                obj.__class__.__name__)
            )
    



"""
.. versionadded:: 0.5

This function generates Levy flight by integration of Levy
alpha-stable distribution (also reffered just as stable distribution).

The Levy distribution is defined by two parameters :math:`\\alpha`
and :math:`\\beta`. The Gaussian distribution is special case of 
Levy distribution with :math:`\\alpha=2` and :math:`\\beta=0`.
In case of Gaussian distribution the Levy flight is Brownian walk.

This function uses :ref:`generators-levy_noise`.

Example Usage
==================

The following example produce 500 samples of Levy flight produced from
Levy noise located (mean value) at 0 (`position`),
with characteristic exponent index of 1.8 (`alpha`),
skeewness of 0 (`beta`) and diffusion of 1. (`sigma`).

.. code-block:: python

    import signalz
    x = signalz.levy_flight(500, alpha=1.8, beta=0., sigma=1., position=0)

Function Documentation
======================================
"""
import numpy as np

import signalz

def levy_flight(n, alpha=2., beta=1., sigma=1., position=0.):
    """
    This function produces Levy flight.
    
    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**
    
    * `alpha` - characteristic exponent index of used Levy noise
      (float) in range `0<alpha<2`
      
    * `beta` - skeewness of used Levy noise (float) in range `-1<beta<1`

    * `sigma` - diffusion of used Levy noise (float),
      in case of gaussian distribution it is
      standard deviation

    * `position` - position parameter (float) of used Levy noise
    
    **Returns:**
    
    * vector of values representing the flight (1d array)
    """
    x = signalz.levy_noise(n, alpha, beta, sigma, position)
    return np.cumsum(x)  
        
        




"""
.. versionadded:: 0.1
.. versionchanged:: 0.4

This function generates Gaussian white noise series. This function uses
`numpy.random.normal`.

Function Documentation
======================================
"""
import numpy as np

def gaussian_white_noise(n, offset=0, std=1):
    """
    Random values with normal distribution.

    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**

    * `offset` - mean value (float)
         
    * `std` - standard deviation (float)
    
    **Returns:**
    
    * vector of values representing the noise (1d array)   
    """
    return np.random.normal(offset, std, n)



# fancy stuff
from signalz.generators.autoregressive_model import autoregressive_model
from signalz.generators.mackey_glass import mackey_glass
from signalz.generators.logistic_map import logistic_map

# noises and walks
from signalz.generators.gaussian_white_noise import gaussian_white_noise
from signalz.generators.uniform_white_noise import uniform_white_noise
from signalz.generators.brownian_noise import brownian_noise
from signalz.generators.levy_noise import levy_noise
from signalz.generators.levy_flight import levy_flight
from signalz.generators.levy_walk import levy_walk

# basic shapes and functions
from signalz.generators.sinus import sinus
from signalz.generators.cosinus import cosinus
from signalz.generators.steps import steps
from signalz.generators.random_steps import random_steps




"""
.. versionadded:: 0.4

This function generates Uniform white noise series. This function uses
`numpy.random.uniform`.

Function Documentation
======================================
"""
import numpy as np

def uniform_white_noise(n, minimum=-1, maximum=1):
    """
    Random values with uniform distribution.

    **Args:**
    
    * `n` - length of the output data (int) - how many samples will be on output
    
    **Kwargs:**

    * `minimum` - minimal value (float)
         
    * `maximum` - maximal value (float)
    
    **Returns:**
    
    * vector of values representing the noise (1d array)   
    """
    return np.random.uniform(low=minimum, high=maximum, size=n)



from distutils.core import setup
from setuptools import find_packages

def readme():
    try:
        with open('README.rst') as f:
            return f.read()
    except:
        pass

setup(
    name = 'signalz',
    packages = find_packages(exclude=("tests",)),
    version = '0.5',
    description = 'Synthetic data generators in Python',
    long_description = readme(),
    author = 'Matous Cejnek',
    maintainer = "Matous Cejnek",
    author_email = 'matousc@gmail.com',
    license = 'MIT',
    url = 'http://matousc89.github.io/signalz/',
    download_url = 'https://github.com/matousc89/signalz/',
    keywords = ['signals', 'data', 'artificial', 'generators', 'synthetic', 'noise'],
    install_requires=[
        'numpy',
    ],    
    bugtrack_url = "https://github.com/matousc89/signalz/issues", 
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Testing',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Programming Language :: Python',
    ],
)
